/*
Module : VIDEOMOD.CPP
Purpose: Implementation for some handy classes which wrap access 
         to the EnumDisplaySettings & ChangeDisplaySettings SDK functions
Created: PJN / 24-09-1997
History: PJN / 03-01-1999 1. Major tidy up of the code
                          2. UNICODE enabled the code
         PJN / 02-04-2007 1. Updated copyright details.
                          2. Updated the documentation to use the same style as the web site.
                          3. Optimized CVideoMode constructor code
                          4. Replaced call to _tprintf in GetCurrentVideoMode with a
                          TRACE call.
                          5. Made the ResSwitch app unicode compliant.
                          6. Updated the code to clean compile on VC 2005
                          7. Now includes support for multiple monitors / display devices. This is
                          achieved through the use of the newly provided CDisplayDevices class
                          8. GetAvailableVideoModes method now allows the device name to be specified
                          for which display modes are to be obtained and the method has been renamed 
                          to just GetAvailable.
                          9. GetCurrentVideoMode has been renamed to the more appropriate
                          name of GetCurrentForPrimaryDisplay
                          10. Removed RevertVideoModeToDefault, ChangeVideoModePermanently,
                          ChangeVideoModeTemporarily, CanChangeVideoMode, ReportChangeVideoErrorValue
                          and CreateCompatibleDEVMODE methods as they did not provide any useful 
                          encapsulation over calling the ChangeDisplaySettings SDK call directly. You 
                          can now think of the various classes in the Videomod module as a simple wrapper
                          for enumerating the display devices and their associated display modes.
                          11. Added a m_dwDisplayFlags value to the CVideoMode class.
                          12. CAvailableVideoModes class is now called CAvailableDisplayModes.
                          13. CVideoMode class is now called CDisplayMode
                          14. CVideoModes class is now called CAvailableDisplayModes
                          15. Updated the documentation to use the same style as the web site.
                          16. Reworked ResCopy example to use a tree control to display all the details
                          about all display devices, their attributes and available display modes
                          17. Addition of version infos to both exes included in the download.
                          18. Reworked ResSwitch application to provide a much more comprehensive set
                          of command line parameters as well a general rework of its codebase.
         PJN / 12-03-2008 1. Updated copyright details.
                          2. Fixed a bug in CMyCommandLineInfo::ParseParam in ResSwitch.cpp where the 
                          /DEVICE command line parameter was being parsed incorrectly. Thanks to Ron Cozad for 
                          reporting this bug.
                          3. Removed VC 6 style comments from the code base.
                          4. Updated the code to clean compile on VC 2008

Copyright (c) 1996 - 2008 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////////////  Includes  //////////////////////////////////

#include "stdafx.h"
#include "Videomod.h"


///////////////////////////////// defines /////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


//////////////////////////////// Implementation ///////////////////////////////

BOOL CAvailableDisplayDevices::Get(CDisplayDevices& devices)
{
  //Empty our the "devices" output parameter
  devices.RemoveAll();

  int i=0;
  DISPLAY_DEVICE dv;
  dv.cb = sizeof(dv);
  while (EnumDisplayDevices(NULL, i, &dv, 0))
  {
    CDisplayDevice display;
    display.m_sDeviceName = dv.DeviceName;
    display.m_sDeviceString = dv.DeviceString;
    display.m_dwStateFlags = dv.StateFlags;
    display.m_sDeviceID = dv.DeviceID;
    devices.SetAtGrow(i, display);
    ++i;
  }

  return TRUE;
}

BOOL CAvailableDisplayModes::GetCurrentForPrimaryDisplay(CDisplayMode& mode)
{
  HDC hdc = ::GetDC(NULL);  //Use the Screen DC used to get current display settings
  if (hdc == NULL)
    return FALSE;

  mode.m_dwBitsPerPixel = GetDeviceCaps(hdc, BITSPIXEL);
  mode.m_dwWidth = GetDeviceCaps(hdc, HORZRES);
  mode.m_dwHeight = GetDeviceCaps(hdc, VERTRES);
  mode.m_dwFrequency = GetDeviceCaps(hdc, VREFRESH);
  mode.m_dwDisplayFlags = 0; //We do not support returing display flags via this method

  ::ReleaseDC(NULL, hdc);

  return TRUE;
}

BOOL CAvailableDisplayModes::GetAvailable(LPCTSTR pszDeviceName, CDisplayModes& modes)
{
  //Empty our the "modes" output parameter
  modes.RemoveAll();

  int i = 0;
  DEVMODE dm;
  while (EnumDisplaySettings(pszDeviceName, i, &dm))
  {
    CDisplayMode mode;
    mode.m_dwBitsPerPixel = dm.dmBitsPerPel;
    mode.m_dwWidth = dm.dmPelsWidth;
    mode.m_dwHeight = dm.dmPelsHeight;
    mode.m_dwFrequency = dm.dmDisplayFrequency;
    mode.m_dwDisplayFlags = dm.dmDisplayFlags;
    modes.SetAtGrow(i, mode);
    ++i;
  }

  return TRUE;
}

